/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_HESAIPOINT_H
#define IMPL_TYPE_HESAIPOINT_H

#include <cfloat>
#include <cmath>
#include "impl_type_float.h"
#include "impl_type_uint32.h"

struct HesaiPoint {
    ::Float x{ 0.0F };
    ::Float y{ 0.0F };
    ::Float z{ 0.0F };
    ::Float intensity{ 0.0F };
    ::UInt32 ring{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(x);
        fun(y);
        fun(z);
        fun(intensity);
        fun(ring);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(x);
        fun(y);
        fun(z);
        fun(intensity);
        fun(ring);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("x", x);
        fun("y", y);
        fun("z", z);
        fun("intensity", intensity);
        fun("ring", ring);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("x", x);
        fun("y", y);
        fun("z", z);
        fun("intensity", intensity);
        fun("ring", ring);
    }

    friend bool operator==(const ::HesaiPoint& lhs, const ::HesaiPoint& rhs) noexcept
    {
        return (fabs(static_cast<double>(lhs.x - rhs.x)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.y - rhs.y)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.z - rhs.z)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.intensity - rhs.intensity)) < DBL_EPSILON) && (lhs.ring == rhs.ring);
    }
};


#endif // IMPL_TYPE_HESAIPOINT_H
